'''Trains a simple convnet on the MNIST dataset.
Gets to 99.25% test accuracy after 12 epochs
(there is still a lot of margin for parameter tuning).
16 seconds per epoch on a GRID K520 GPU.
'''

from __future__ import print_function
import keras
from keras.datasets import mnist
from keras.models import Sequential
from keras.layers import Dense, Dropout, Flatten
from keras.layers import Conv2D, MaxPooling2D
from keras import backend as K
import matplotlib.pyplot as plt

batch_size = 128
epochs = 40
num_classes = 10

# the data, split between train and test sets

import numpy as np
from keras.datasets import mnist

(x, y), (x_test, y_test) = mnist.load_data()
y_val = y[0:10000]
y_train = y[10000:]
x_val = np.reshape(x[0:10000],(10000,28,28,1)).\
            astype(np.float32)/255
x_train = np.reshape(x[10000:],(50000,28,28,1)).\
              astype(np.float32)/255

x_test = np.reshape(x_test,(10000,28,28,1)).\
              astype(np.float32)/255




# convert class vectors to binary class matrices
y_train = keras.utils.to_categorical(y_train, num_classes)
y_val = keras.utils.to_categorical(y_val, num_classes)
y_test = keras.utils.to_categorical(y_test, num_classes)


model = Sequential()
model.add(Conv2D(64, (3,3), activation='relu'))
model.add(MaxPooling2D(pool_size=(2,2)))
model.add(Conv2D(32, (3,3), activation='relu'))
model.add(MaxPooling2D(pool_size=(2,2)))
model.add(Dropout(0.25))
model.add(Flatten())
model.add(Dense(30, activation='relu'))
model.add(Dropout(0.25))
model.add(Dense(10, activation='softmax'))


callbacks = []
checkpoint = keras.callbacks.ModelCheckpoint(
    'mnist_{epoch:02d}-{val_loss:.8f}-{val_acc:.4f}.hdf5',
    verbose=1)
callbacks.append(checkpoint)

tb_callback = keras.callbacks.TensorBoard(
    log_dir='./logs')
callbacks.append(tb_callback)


loss = keras.losses.categorical_crossentropy
optimizer = keras.optimizers.Adam()
model.compile(loss=loss,
              optimizer=optimizer,
              metrics=['accuracy'])

model.fit(x_train, y_train,
          batch_size=batch_size,
          epochs=epochs,
          verbose=1,
          validation_data=(x_val, y_val),
          callbacks = callbacks)
score = model.evaluate(x_test, y_test)
print('Test loss:', score[0])
print('Test accuracy:', score[1])